###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Machine Learning Training
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
"""
Base implementation of this pipeline.

GENERATED CODE - DO NOT MODIFY (add your customizations in ExampleMachineLearningPipeline).

Generated from: templates/general-mlflow/training.base.py.vm
"""
from abc import ABC, abstractmethod
from typing import List, NamedTuple
from pandas import DataFrame
from ..config.pipeline_config import PipelineConfig
from datetime import datetime
import mlflow
import json
from aissemble_security.pdp_client import PDPClient
from aissembleauth.auth_config import AuthConfig
from pathlib import Path
from krausening.logging import LogManager

from ..post_action.example_onnx_sklearn_conversion import ExampleOnnxSklearnConversion
from ..post_action.example_onnx_keras_conversion import ExampleOnnxKerasConversion
from ..post_action.example_custom_conversion import ExampleCustomConversion
from ..post_action.example_freeform_post_action import ExampleFreeformPostAction

from .pipeline.pipeline_base import PipelineBase
from uuid import uuid4
from aissemble_data_lineage import Run, Job, Emitter, RunEvent, InputDataset, from_open_lineage_facet, LineageUtil, LineageEventData
from aissemble_model_lineage import MLflowRunFacet, LineageBuilder
from aissemble_core_config import MessagingConfig
import os
import attr

from openlineage.client.facet import ErrorMessageRunFacet, NominalTimeRunFacet, ParentRunFacet
from platform import python_version


class DatasetSplits(NamedTuple):
    """
    Class to store the training and test splits of a dataset.
    The splits are of type any, to allow for custom implementation
    for handling any number of datasets per split.
    """
    train: any
    test: any

class ExampleMachineLearningPipelineBase(ABC):
    """
    Base implementation of the pipeline.
    """
    logger = LogManager.get_instance().get_logger('ExampleMachineLearningPipeline')

    def __init__(self, experiment_name):
        """
        Default initializations for the pipeline.
        """
        # set default mlflow configurations
        self.config = PipelineConfig()
        mlflow.set_tracking_uri(self.config.mlflow_tracking_uri())
        mlflow.set_experiment(experiment_name)

        self.messaging_config = MessagingConfig()
        self.emitter = Emitter()
        self.lineage_builder = LineageBuilder()
        self.lineage_util = LineageUtil()


    @abstractmethod
    def acknowledge_training_alert(self, alert: any) -> None:
        """
        Method to acknowledge a training alert for auto-training purposes.
        """
        pass


    @abstractmethod
    def load_dataset(self) -> DataFrame:
        """
        Method to load a dataset for training.
        Returns a dataset of type DataFrame.
        """
        pass


    @abstractmethod
    def prep_dataset(self, dataset: DataFrame) -> DataFrame:
        """
        Method to perform last-mile data preparation on the loaded dataset.
        Returns the prepped dataset.
        """
        pass


    @abstractmethod
    def select_features(self, dataset: DataFrame) -> List[str]:
        """
        Method to perform feature selection on the prepped dataset.
        Returns a list of the features (columns) selected from the dataset.
        """
        pass


    @abstractmethod
    def split_dataset(self, dataset: DataFrame) -> DatasetSplits:
        """
        Method to create the train and test splits on the dataset with selected features.
        Returns the splits within a DatasetSplits object.
        """
        pass


    @abstractmethod
    def train_model(self, train_dataset: any) -> any:
        """
        Method to train a model with the training dataset split(s).
        Returns the model that has been trained.
        """
        pass


    @abstractmethod
    def evaluate_model(self, model: any, test_dataset: any) -> float:
        """
        Method to evaluate the trained model with the test dataset split(s).
        Returns the score of the model evaluation.
        """
        pass


    @abstractmethod
    def save_model(self, model: any) -> None:
        """
        Method to save the model to a location.
        """
        pass


    @abstractmethod
    def deploy_model(self, score: float, model: any) -> None:
        """
        Method to deploy the model if needed.
        """
        pass


    def run(self):
        """
        Runs the pipeline.
        """
        self.logger.info('Running %s...' % type(self).__name__)

        run_id = uuid4()
        parent_run_facet = PipelineBase().get_pipeline_run_as_parent_run_facet()
        job_name= self.get_job_name()
        try:
            with mlflow.start_run() as run:
                self.training_run_id = run.info.run_uuid
                # pylint: disable-next=assignment-from-none
                event_data = self.create_base_lineage_event_data()
                start = datetime.utcnow()
                self.record_lineage(self.create_lineage_start_event(run_id=run_id, job_name=job_name, parent_run_facet=parent_run_facet, event_data=event_data, start_time=start))
                loaded_dataset = self.load_dataset()
                prepped_dataset = self.prep_dataset(loaded_dataset)
                features = self.select_features(prepped_dataset)
                splits = self.split_dataset(prepped_dataset[features])
                model = self.train_model(splits.train)
                score = self.evaluate_model(model, splits.test)
                self.save_model(model)
                self.deploy_model(score, model)
                self.apply_post_actions(self.training_run_id, model)
                end = datetime.utcnow()
                self.log_information(start, end, loaded_dataset, features)
                self.logger.info('Complete')
                self.record_lineage(self.create_lineage_complete_event(run_id=run_id, job_name=job_name, parent_run_facet=parent_run_facet, event_data=event_data, start_time=start, end_time=end))

        except Exception as error:
            self.record_lineage(self.create_lineage_fail_event(run_id=run_id, job_name=job_name, event_data=event_data, parent_run_facet=parent_run_facet, start_time=start, end_time=datetime.now(), error=error))
            PipelineBase().record_pipeline_lineage_fail_event()
            raise Exception(error)

    def apply_post_actions(self, training_run_id: str, model: any) -> None:
        """
        Applies the post actions specified for the training.
        """
        postActionExampleOnnxSklearnConversion = ExampleOnnxSklearnConversion(training_run_id, model)
        postActionExampleOnnxSklearnConversion.apply()

        postActionExampleOnnxKerasConversion = ExampleOnnxKerasConversion(training_run_id, model)
        postActionExampleOnnxKerasConversion.apply()

        postActionExampleCustomConversion = ExampleCustomConversion(training_run_id, model)
        postActionExampleCustomConversion.apply()

        postActionExampleFreeformPostAction = ExampleFreeformPostAction(training_run_id, model)
        postActionExampleFreeformPostAction.apply()

    def create_base_lineage_event_data(self) -> LineageEventData:
        """
        Create a base lineage event data that will included in all the step events

        Returns
            LineageEventData
        """
        job_facets = {
            "documentation": from_open_lineage_facet(self.lineage_builder.get_documentation_job_facet()),
            "ownership": from_open_lineage_facet(self.lineage_builder.get_ownership_job_facet()),
            "sourceCodeLocation": from_open_lineage_facet(self.lineage_builder.get_source_code_directory_job_facet())
        }
        run_facets = {
            "hardwareDetails": from_open_lineage_facet(self.lineage_builder.get_hardware_details_run_facet()),
            "hyperparameters": from_open_lineage_facet(self.lineage_builder.get_hyperparameter_run_facet()),
            "mlflowRunId": from_open_lineage_facet(MLflowRunFacet(self.training_run_id)),
            "performanceMetrics": from_open_lineage_facet(self.lineage_builder.get_performance_metric_run_facet())
        }
        dataset_facets = {
            "dataSource": from_open_lineage_facet(self.lineage_builder.get_data_source_dataset_facet()),
            "dataQualityAssertions": from_open_lineage_facet(self.lineage_builder.get_data_quality_assertions_facet()),
            "ownership": from_open_lineage_facet(self.lineage_builder.get_ownership_dataset_facet()),
            "schema": from_open_lineage_facet(self.lineage_builder.get_schema_dataset_facet()),
            "storage": from_open_lineage_facet(self.lineage_builder.get_storage_dataset_facet())
        }
        input_dataset = InputDataset("ExampleMachineLearningPipelineInput", dataset_facets)

        return LineageEventData(job_facets=job_facets, run_facets=run_facets, event_inputs=[input_dataset])

    def create_lineage_start_event(self, run_id: str = None, job_name: str = "", parent_run_facet: ParentRunFacet = None, event_data: LineageEventData = None, **kwargs) -> RunEvent:
        """
        Creates the Start RunEvent with given uuid, parent run facet, job name, lineage data event or any input parameters
        To customize the event, override the customize_lineage_start_event(...) function to include the job facets, run facets
        or the inputs/outputs dataset.

        The customize_run_event(..) is deprecated customize point.

        Returns:
            RunEvent created from the input arguments
        """

        event = self.lineage_util.create_start_run_event(
            run_id=run_id,
            parent_run_facet=parent_run_facet,
            job_name=job_name,
            event_data=event_data)
        event = self.customize_lineage_start_event(event, **kwargs)
        return self.customize_run_event(event)

    def customize_lineage_start_event(self, event: RunEvent = None, **kwargs) -> RunEvent:
        """
        Customize the start event with the given input

        Returns
            lineage event
        """

        if "start_time" in kwargs:
            run_facets = {
                "nominalTime": from_open_lineage_facet(NominalTimeRunFacet(kwargs["start_time"].isoformat(timespec="milliseconds") + "Z"))
            }
            event.run.facets.update(run_facets)

        return event

    def create_lineage_complete_event(self, run_id: str = None, job_name: str = "", parent_run_facet: ParentRunFacet = None, event_data: LineageEventData = None, **kwargs) -> RunEvent:
        """
        Creates the Complete RunEvent with given uuid, parent run facet, job name, lineage data event or any input parameters
        To customize the event, override the customize_lineage_complete_event(...) function to include the job facets, run facets
        or the inputs/outputs dataset.

        The customize_run_event(...) is deprecated customize point.

        Returns:
            RunEvent created from the input arguments
        """

        event = self.lineage_util.create_complete_run_event(
            run_id=run_id,
            parent_run_facet=parent_run_facet,
            job_name=job_name,
            event_data=event_data)
        event = self.customize_lineage_complete_event(event, **kwargs)
        return self.customize_run_event(event)

    def customize_lineage_complete_event(self, event: RunEvent = None, **kwargs) -> RunEvent:
        """
        Customize the complete event with the given input

        Returns
            lineage event
        """

        if "start_time" in kwargs and "end_time" in kwargs:
           event.run.facets.update(self.record_run_end(kwargs["start_time"], kwargs["end_time"]))
        return event

    def create_lineage_fail_event(self, run_id: str = None, job_name: str = "", parent_run_facet: ParentRunFacet = None, event_data: LineageEventData = None, **kwargs) -> RunEvent:
        """
        Creates the Fail RunEvent with given uuid, parent run facet, job name, lineage data event or any input parameters
        To customize the event, override the customize_lineage_fail_event(...) function to include the job facets, run facets
        or the inputs/outputs dataset.

        The customize_run_event() is deprecated customize point.

        Returns:
            RunEvent created from the input arguments
        """

        event = self.lineage_util.create_fail_run_event(
            run_id=run_id,
            parent_run_facet=parent_run_facet,
            job_name=job_name,
            event_data=event_data)
        event = self.customize_lineage_fail_event(event, **kwargs)
        return self.customize_run_event(event)

    def customize_lineage_fail_event(self, event: RunEvent = None, **kwargs) -> RunEvent:
        """
        Customize the fail event with the given input

        Returns
            lineage event
        """

        if "start_time" in kwargs and "end_time" in kwargs and "error" in kwargs:
           event.run.facets.update(self.record_run_end(kwargs["start_time"], kwargs["end_time"], kwargs["error"]))
        return event

    def customize_run_event(self, event: RunEvent) -> RunEvent:
        """
        Override this method to modify the created RunEvent.  Provides an opportunity for adding customizations,
        such as Input or Output Datasets.

        Returns:
            RunEvent with customizations added.
        """

        return event

    def record_run_end(self, start_time: datetime, end_time: datetime, error: Exception = None) -> None:
        """
        Records the end of the training run by updating the OpenLineage Run.  The end of the run can be due to successful
        completion of the logic or by an error.

        :param start_time: The start time of the training execution.
        :param end_time: The end time of the training execution.
        :param error: The `Exception` that caused the run to fail, if applicable. `None` if the run was successful.
        """

        run_end = { "nominalTime": from_open_lineage_facet(NominalTimeRunFacet(start_time.isoformat(timespec="milliseconds") + "Z", end_time.isoformat(timespec="milliseconds") + "Z"))}
        if error:
            run_end.update({"errorMessage": from_open_lineage_facet(ErrorMessageRunFacet(str(error), "Python"+python_version()))})

        return run_end

    def record_lineage(self, event: RunEvent):
        """
        Records metadata for this step in an OpenLineage format.
        """

        self.lineage_util.record_lineage(self.emitter, event)

    def get_job_name(self) -> str:
        """
        The default job name is the training step name; override this function to change the default job name
        """
        return "ExampleMachineLearningPipeline.AissembleMachineLearningTraining"


    def set_dataset_origin(self, origin: str) -> None:
        """
        Sets the origin of the dataset for a training run.
        """
        if not origin:
            self.logger.warning('No value given for dataset origin!')

        self.dataset_origin = origin


    def set_model_information(self, model_type: str, model_architecture: str) -> None:
        """
        Sets the model information for a training run.
        """
        if not model_type:
            self.logger.warning('No value given for model type!')
        if not model_architecture:
            self.logger.warning('No value given for model architecture!')

        self.model_type = model_type
        self.model_architecture = model_architecture


    def log_information(self, start: datetime, end: datetime, loaded_dataset: DataFrame, selected_features: List[str]) -> None:
        """
        Log information into MLflow tags.
        """
        try:
            mlflow.set_tags(
                {
                    "architecture": self.model_architecture,
                    "dataset_origin": self.dataset_origin,
                    "dataset_size": len(loaded_dataset),
                    "end_time": end,
                    "original_features": list(loaded_dataset),
                    "selected_features": selected_features,
                    "start_time": start,
                    "type": self.model_type,
                }
            )
        except Exception as error:
            raise Exception(error)


    def authorize(self, token: str, action: str):
        """
        Calls the Policy Decision Point server to authorize a jwt
        """

        auth_config = AuthConfig()

        pdp_client = PDPClient(auth_config.pdp_host_url())

        decision = pdp_client.authorize(token, "", action)

        return decision



###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Machine Learning Training
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
from abc import ABC, abstractmethod
from ...config.pipeline_config import PipelineConfig
from krausening.logging import LogManager


class CustomModelConversionBase(ABC):
    """
    Base custom model conversion post-action class.

    GENERATED CODE - DO NOT MODIFY (add your customizations in ExampleCustomConversion).

    Generated from: templates/post-action/model.conversion.base.py.vm
    """
    logger = LogManager.get_instance().get_logger('CustomModelConversionBase')


    def __init__(self) -> None:
        """
        Default constructor for a custom model conversion post-action.
        """
        self._pipeline_config = PipelineConfig()


    @property
    def converted_model_file_directory(self) -> str:
        """
        The directory to save the converted model to.

        :return: the directory so save the converted model to
        """
        return self._pipeline_config.onnx_model_directory()


    @property
    def converted_model_file_name(self) -> str:
        """
        The file name to save the converted model as.

        :return: the file name to save the converted model as
        """
        return 'converted_foo_model.onnx'


    @abstractmethod
    def _convert(self, source_model) -> any:
        """
        Performs the foo-to-custom conversion on the source model and returns the converted model.

        :source_model: the model to perform the conversion on
        :return: the converted model
        """
        pass


    @abstractmethod
    def _save(self, converted_model) -> None:
        """
        Saves the converted custom model.

        :converted_model: the converted model to save
        """
        pass





###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Machine Learning Training
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
from ...generated.post_action.onnx_keras_model_conversion_base import OnnxKerasModelConversionBase
from krausening.logging import LogManager


class ExampleOnnxKerasConversionBase(OnnxKerasModelConversionBase):
    """
    Base ExampleOnnxKerasConversion post-action class.

    GENERATED CODE - DO NOT MODIFY (add your customizations in ExampleOnnxKerasConversion).

    Generated from: templates/post-action/post.action.base.py.vm
    """
    logger = LogManager.get_instance().get_logger('ExampleOnnxKerasConversionBase')


    def __init__(self, training_run_id: str, model: any) -> None:
        """
        Default constructor for this post-action.

        :model: the model to apply this post-action on
        :training_run_id: the training run identifier associated with this post-action
        """
        super().__init__()
        self._training_run_id = training_run_id
        self._model = model


    @property
    def training_run_id(self) -> str:
        """
        The training run identifier associated with this post-action.

        :return: the training run identifier associated with this post-action.
        """
        return self._training_run_id


    @property
    def model(self) -> any:
        """
        The model to apply this post-action on.

        :return: the model to apply this post-action on
        """
        return self._model


    def apply(self) -> None:
        """
        Applies this model-conversion post-action.
        """
        ExampleOnnxKerasConversionBase.logger.info('Applying model conversion post action...')

        converted_model = self._convert(self.model)
        self._save(converted_model)

        ExampleOnnxKerasConversionBase.logger.info('Applied model conversion post action')



###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Machine Learning Training
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
from abc import ABC, abstractmethod
from ...config.pipeline_config import PipelineConfig
from krausening.logging import LogManager
from onnxmltools.convert import convert_keras
import mlflow
from os import path


class OnnxKerasModelConversionBase(ABC):
    """
    Base onnx model conversion post-action class.

    GENERATED CODE - DO NOT MODIFY (add your customizations in ExampleOnnxKerasConversion).

    Generated from: templates/post-action/model.conversion.base.py.vm
    """
    logger = LogManager.get_instance().get_logger('OnnxKerasModelConversionBase')


    def __init__(self) -> None:
        """
        Default constructor for a onnx model conversion post-action.
        """
        self._pipeline_config = PipelineConfig()


    @property
    def converted_model_file_directory(self) -> str:
        """
        The directory to save the converted model to.

        :return: the directory so save the converted model to
        """
        return self._pipeline_config.onnx_model_directory()


    @property
    def converted_model_file_name(self) -> str:
        """
        The file name to save the converted model as.

        :return: the file name to save the converted model as
        """
        return 'converted_keras_model.onnx'


    def _convert(self, source_model) -> any:
        """
        Performs the keras-to-onnx conversion on the source model and returns the converted model.

        :source_model: the model to perform the conversion on
        :return: the converted model
        """
        OnnxKerasModelConversionBase.logger.info('Performing onnx model conversion...')

        converted_model = convert_keras(
            source_model,
            custom_shape_calculators=self.custom_shape_calculators,
            custom_conversion_functions=self.custom_conversion_functions,
            name=self.name,
            target_opset=self.target_opset,
            channel_first_inputs=self.channel_first_inputs,
            default_batch_size=self.default_batch_size,
            initial_types=self.initial_types,
            doc_string=self.doc_string,
        )

        OnnxKerasModelConversionBase.logger.info('Converted keras model to onnx format')
        return converted_model


    def _save(self, converted_model) -> None:
        """
        Saves the converted onnx model.

        :converted_model: the converted model to save
        """
        OnnxKerasModelConversionBase.logger.info('Saving converted onnx model...')

        mlflow.onnx.log_model(converted_model, self.converted_model_file_directory)

        OnnxKerasModelConversionBase.logger.info('Saved converted onnx model to {}'.format(self.converted_model_file_directory))


    @property
    def custom_shape_calculators(self):
        """
        Optional custom_shape_calculators parameter for the keras-to-onnx conversion.

        :return: optional custom_shape_calculators parameter value
        """
        return None


    @property
    def custom_conversion_functions(self):
        """
        Optional custom_conversion_functions parameter for the keras-to-onnx conversion.

        :return: optional custom_conversion_functions parameter value
        """
        return None


    @property
    def name(self):
        """
        Optional name parameter for the keras-to-onnx conversion.

        :return: optional name parameter value
        """
        return None


    @property
    def target_opset(self):
        """
        Optional target_opset parameter for the keras-to-onnx conversion.

        :return: optional target_opset parameter value
        """
        return None


    @property
    def channel_first_inputs(self):
        """
        Optional channel_first_inputs parameter for the keras-to-onnx conversion.

        :return: optional channel_first_inputs parameter value
        """
        return None


    @property
    def default_batch_size(self):
        """
        Optional default_batch_size parameter for the keras-to-onnx conversion.

        :return: optional default_batch_size parameter value
        """
        return 1


    @property
    def initial_types(self):
        """
        Optional initial_types parameter for the keras-to-onnx conversion.

        :return: optional initial_types parameter value
        """
        return None


    @property
    def doc_string(self):
        """
        Optional doc_string parameter for the keras-to-onnx conversion.

        :return: optional doc_string parameter value
        """
        return ''





###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Machine Learning Training
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
from krausening.logging import LogManager
from openlineage.client.facet import ParentRunFacet
from uuid import uuid4, UUID
from aissemble_data_lineage import Emitter, LineageUtil

class PipelineBase:
    """
    Performs pipeline level process for ExampleMachineLearningPipeline.

    GENERATED CODE - DO NOT MODIFY

    Generated from: templates/pipeline.base.py.vm
    """

    _instance = None
    _pipeline_run_id: UUID = None
    _pipeline_job_namespace: str = "ExampleMachineLearningPipeline"
    _pipeline_name: str = "ExampleMachineLearningPipeline"
    _pipeline_event_started = False
    _emitter: Emitter = None
    _lineage_util = LineageUtil()
    logger = LogManager.get_instance().get_logger('PipelineBase')


    def __new__(cls):
        """
        Create a singleton class for pipeline level process
        """
        if cls._instance is None:
            print("Creating the PipelineBase")
            cls._instance = super(PipelineBase, cls).__new__(cls)
        return cls._instance

    def record_pipeline_lineage_complete_event(self):
        """
        Record a pipeline lineage Complete event
        """
        if self._pipeline_event_started:
            if self._emitter is None:
                self._emitter = Emitter()

            self._lineage_util.record_lineage(self._emitter, self._lineage_util.create_complete_run_event(run_id=self.get_pipeline_run_id(), job_name=self._pipeline_name, default_namespace=self._pipeline_job_namespace))
            self._pipeline_event_started = False
            self._pipeline_run_id = None
            self.logger.info('Complete pipeline job run..')
        else:
            self.logger.warn("Pipeline hasn't recorded a lineage start event")

    def record_pipeline_lineage_fail_event(self):
        """
        Record a pipeline lineage Fail event
        """
        if self._pipeline_event_started:
            if self._emitter is None:
                self._emitter = Emitter()

            self._lineage_util.record_lineage(self._emitter, self._lineage_util.create_fail_run_event(run_id=self.get_pipeline_run_id(), job_name=self._pipeline_name, default_namespace=self._pipeline_job_namespace))
            self._pipeline_event_started = False
            self._pipeline_run_id = None
            self.logger.info('Fail pipeline job run..')
        else:
            self.logger.warn("Pipeline hasn't recorded a lineage start event")


    def record_pipeline_lineage_start_event(self):
        """
        Record a pipeline lineage Start event
        """
        if self._emitter is None:
            self._emitter = Emitter()

        # always make sure one job run at a time
        if not self._pipeline_event_started:
            run_event = self._lineage_util.create_start_run_event(run_id=self.get_pipeline_run_id(), job_name=self._pipeline_name, default_namespace=self._pipeline_job_namespace)
            self._pipeline_job_namespace = run_event.job.get_open_lineage_job().namespace
            self._lineage_util.record_lineage(self._emitter, run_event)
            self._pipeline_event_started = True
            self.logger.info('Start pipeline job run..')
        else:
            self.logger.warn("Pipeline has recorded a lineage start event");



    def get_pipeline_run_id(self) -> UUID:
        """
        Get the pipeline run id

        Returns:
            pipeline run id
        """
        if self._pipeline_run_id is None:
            self._pipeline_run_id = uuid4()
        return self._pipeline_run_id


    def get_pipeline_name(self) -> str:
        """
        Get the pipeline name

        Returns:
            pipeline name
        """
        return self._pipeline_name


    def get_pipeline_job_namespace(self) -> str:
        """
        Get the pipeline Job's namespace

        Returns:
            Namespace for the pipeline job
        """

        return self._pipeline_job_namespace


    def get_pipeline_run_as_parent_run_facet(self) -> ParentRunFacet:
        """
        Get the pipeline run event information as a ParentRunFacet

        Returns:
            ParentRunFacet created from pipeline run event information
        """

        if self._pipeline_run_id is not None:
            return ParentRunFacet(
                    run={"runId": str(self._pipeline_run_id)},
                    job={
                        "namespace": self._pipeline_job_namespace,
                        "name": self._pipeline_name,
                    },
                )








###
# #%L
# aiSSEMBLE::Test::MDA::Machine Learning::Machine Learning Training
# %%
# Copyright (C) 2021 Booz Allen
# %%
# This software package is licensed under the Booz Allen Public License. All Rights Reserved.
# #L%
###
"""
Base implementation of this pipeline.

GENERATED CODE - DO NOT MODIFY (add your customizations in Bootstrap).

Generated from: templates/pipeline-common/generic.base.py.vm
"""

from abc import ABC

class BootstrapBase(ABC):
    def __init__(self):
        pass
    
